//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "FormMain.h"
#include "FormAbout.h"
#include "FormSettings.h"
#include "LogUnit.h"
#include "Log.h"
#include "WaitUnit.h"
#include "UaMain.h"
#include "ControlQueue.h"
#include "Callback.h"
#include "CallbackQueue.h"
#include <assert.h>

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TfrmMain *frmMain;
//---------------------------------------------------------------------------
__fastcall TfrmMain::TfrmMain(TComponent* Owner)
	: TForm(Owner)
{
	prevSelectedAcc = NULL;
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::FormCreate(TObject *Sender)
{
	AnsiString asConfigFile = ChangeFileExt( Application->ExeName, ".json" );
	appSettings.Read(asConfigFile);
	if (this->BorderStyle != bsSingle)
	{
		this->Width = appSettings.frmMain.iWidth;
		this->Height = appSettings.frmMain.iHeight;
	}
	this->Top = appSettings.frmMain.iPosY;
	this->Left = appSettings.frmMain.iPosX;
	if (appSettings.frmMain.bAlwaysOnTop)
		this->FormStyle = fsStayOnTop;
	else
		this->FormStyle = fsNormal;
	if (appSettings.frmMain.bWindowMaximized)
		this->WindowState = wsMaximized;
	if (appSettings.Logging.bLogToFile)
		CLog::Instance()->SetFile(ChangeFileExt(Application->ExeName, ".log").c_str());
	else
		CLog::Instance()->SetFile("");
	UpdateCallHistory();
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::FormCloseQuery(TObject *Sender, bool &CanClose)
{
	AnsiString asConfigFile = ChangeFileExt( Application->ExeName, ".json" );
	appSettings.frmMain.bWindowMaximized = (this->WindowState == wsMaximized);
	if (!appSettings.frmMain.bWindowMaximized)
	{
		// these values are meaningless is wnd is maximized
		appSettings.frmMain.iWidth = this->Width;
		appSettings.frmMain.iHeight = this->Height;
		appSettings.frmMain.iPosY = this->Top;
		appSettings.frmMain.iPosX = this->Left;
	}
	appSettings.Write(asConfigFile);

	CanClose = true;	
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::actShowAboutExecute(TObject *Sender)
{
	frmAbout->ShowModal();	
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::actShowSettingsExecute(TObject *Sender)
{
    UaConf tmp = appSettings.uaConf;
	frmSettings->ShowModal();
	if (appSettings.uaConf != tmp)
	{
        StatusBar->SimpleText = "Restarting UA...";
		UaMain::Instance().Restart();

		reg_state.clear();
		reg_state.resize(appSettings.uaConf.accounts.size());
		lvAccounts->Items->Count = appSettings.uaConf.accounts.size();
		lvAccounts->Invalidate();
	}
	if (appSettings.uaConf.logMessages != tmp.logMessages)
	{
		UA->SetMsgLogging(appSettings.uaConf.logMessages);
	}
	if (appSettings.frmMain.bAlwaysOnTop)
		this->FormStyle = fsStayOnTop;
	else
		this->FormStyle = fsNormal;
	if (appSettings.Logging.bLogToFile)
		CLog::Instance()->SetFile(ChangeFileExt(Application->ExeName, ".log").c_str());
	else
		CLog::Instance()->SetFile("");
	frmLog->SetLogLinesLimit(appSettings.Logging.iMaxUiLogLines);
	if (frmLog->Visible)
	{
    	frmLog->UpdateUi();
	}

	AnsiString asConfigFile = ChangeFileExt( Application->ExeName, ".json" );
	appSettings.Write(asConfigFile);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::FormShow(TObject *Sender)
{
    static bool once = false;
    if (!once)
    {
		once = true;
		CLog::Instance()->SetLevel(E_LOG_TRACE);
		CLog::Instance()->callbackLog = frmLog->OnLog;
#if 0
		frmLog->SetLogLinesLimit(appSettings.Logging.iMaxUiLogLines);
		frmLog->BorderStyle = bsNone;
		frmLog->Align = alClient;
		frmLog->Parent = tsLog;
		frmLog->Visible = true;
#endif
	}
	LOG("\n===================\nApplication started\n");
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::FormDestroy(TObject *Sender)
{
	UA->Destroy();
	UA_CB->Destroy();
	CLog::Instance()->Destroy();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::actShowLogExecute(TObject *Sender)
{
	if (!frmLog->Visible)
		frmLog->Show();
	frmLog->BringToFront();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::tmrStartupTimer(TObject *Sender)
{
	tmrStartup->Enabled = false;

	UaMain::Instance().Start();
	reg_state.clear();
	reg_state.resize(appSettings.uaConf.accounts.size());
	lvAccounts->Items->Count = appSettings.uaConf.accounts.size();
	lvAccounts->Invalidate();
	if (lvAccounts->Items->Count)
		lvAccounts->Selected = lvAccounts->Items->Item[0];
}
//---------------------------------------------------------------------------

void TfrmMain::UpdateCallHistory(void)
{
	cbCallURI->Clear();
	for (unsigned int i=0; i<appSettings.uaConf.callHistory.size(); i++)
	{
		UaConf::HistoryEntry &entry = appSettings.uaConf.callHistory[i];
		cbCallURI->Items->Add(entry.uri.c_str());
	}
}

void __fastcall TfrmMain::btnMakeCallClick(TObject *Sender)
{
	AnsiString target = cbCallURI->Text.Trim();
	if (target != "")
	{
		if (!lvAccounts->Selected)
			return;
		int accId = lvAccounts->Selected->Index;
		UaCalls &uaCalls = UaMain::Instance().uaCalls;
		int callId = uaCalls.AllocCallId();
		UA->Call(accId, callId, target);
		uaCalls.AllocCall(callId, accId, target, UaCalls::OUTGOING, Callback::CALL_STATE_TRYING);
		lvCalls->Items->Count = uaCalls.calls.size();
		lvCalls->Invalidate();

		UaConf::HistoryEntry entry;
		entry.uri = target.c_str();
		appSettings.uaConf.AddCallHistoryEntry(entry);
		UpdateCallHistory();
	}
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::btnHangupClick(TObject *Sender)
{
	if (!lvCalls->Selected)
		return;
	int id = lvCalls->Selected->Index;
	UaCalls &uaCalls = UaMain::Instance().uaCalls;
	UA->Hangup(uaCalls.calls[id].callId);
}
//---------------------------------------------------------------------------

AnsiString TfrmMain::CleanUri(AnsiString uri)
{
	AnsiString res = uri;
	int start = uri.Pos("<");
	if (start == 0)
		start = 1;
	int end = uri.Pos(">");
	if (end == 0 || (uri.Pos(";") < end))
		end = uri.Pos(";");
	if (start && end && end > start)
	{
		res = uri.SubString(start, end-start);
	}
	return res;
}

void __fastcall TfrmMain::tmrCallbackPollTimer(TObject *Sender)
{
	Callback cb;
	if (UA_CB->GetCallback(cb) != 0)
	{
    	return;
	}
	switch (cb.type)
	{
		case Callback::CALL_STATE:
		{
			UaCalls &uaCalls = UaMain::Instance().uaCalls;
			AnsiString asStateText;
			switch(cb.state)
			{
			case Callback::CALL_STATE_INCOMING:
				uaCalls.AllocCall(cb.callId, cb.accountId, cb.caller, UaCalls::INCOMING, cb.state);
				lvCalls->Items->Count = uaCalls.calls.size();
				break;
			case Callback::CALL_STATE_RINGING:
			case Callback::CALL_STATE_PROGRESS:
			case Callback::CALL_STATE_ESTABLISHED:
			case Callback::CALL_STATE_CLOSED:
				uaCalls.SetState(cb.callId, cb.state);
				break;
			default:
				assert(!"Unhandled call state");
				break;
			}
			lvCalls->Invalidate();
			break;
		}
		case Callback::REG_STATE:
		{
			reg_state[cb.accountId] = cb.reg_state;
			lvAccounts->Invalidate();
			break;
		}
		case Callback::APP_STATE:
		{
			switch(cb.app_state)
			{
			case Callback::APP_START_FAILED:
				StatusBar->SimpleText = "Failed to start application";
				break;
			case Callback::APP_STARTED:
				StatusBar->SimpleText = "Ready";
				break;
			default:
				assert(!"Unhandled app state");
				break;
			}
			break;
		}
		default:
		{
			assert(!"Unhandled callback type");
		}
	}
}
//---------------------------------------------------------------------------

char TfrmMain::TagToDigit(int tag)
{
	char digit;
	switch(tag)
	{
	case 0:
	case 1:
	case 2:
	case 3:
	case 4:
	case 5:
	case 6:
	case 7:
	case 8:
	case 9:
		digit = '0' + tag;
		break;
	case 10:
		digit = '*';
		break;
	case 11:
		digit = '#';
		break;
	default:
		assert(!"Unhandled button tag value");
	}
	return digit;
}

void __fastcall TfrmMain::btnDialClick(TObject *Sender)
{
	ActiveControl = NULL;	// remove focus frame
	char digit = '\0';
	TButton* btn = dynamic_cast<TButton*>(Sender);
	assert(btn);
	digit = TagToDigit(btn->Tag);

	cbCallURI->Text = cbCallURI->Text + digit;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::btnCallDialClick(TObject *Sender)
{
	ActiveControl = NULL;	// remove focus frame

	char digit = '\0';
	TButton* btn = dynamic_cast<TButton*>(Sender);
	assert(btn);
	digit = TagToDigit(btn->Tag);

	if (!lvCalls->Selected)
		return;
	int id = lvCalls->Selected->Index;
	UaCalls &uaCalls = UaMain::Instance().uaCalls;
	if (uaCalls.calls[id].state == Callback::CALL_STATE_ESTABLISHED)
	{
		UA->SendDigit(uaCalls.calls[id].callId, digit);
	}
}
//---------------------------------------------------------------------------


void __fastcall TfrmMain::btnBackspaceClick(TObject *Sender)
{
	cbCallURI->Text = cbCallURI->Text.SubString(1, cbCallURI->Text.Length() - 1);
	cbCallURI->SelStart = cbCallURI->Text.Length();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::FormClose(TObject *Sender, TCloseAction &Action)
{
	StatusBar->SimpleText = "Shutting down...";
	this->Enabled = false;
	frmWait->Show("Unregistering and shutting down...");
	UaMain::Instance().Quit();

	while (!UaMain::Instance().isTerminated())
	{
		Application->ProcessMessages();
		Sleep(10);
	}
	frmWait->Close();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::lvAccountsData(TObject *Sender, TListItem *Item)
{
	int id = Item->Index;
	if (id >= reg_state.size())
	{
		Item->Caption = "";
		Item->SubItems->Add("");
		Item->SubItems->Add("");
		return;
	}
	Item->Caption = id+1;
	AnsiString asRegText;
	if (appSettings.uaConf.accounts.size())
	{
		if (appSettings.uaConf.accounts[id].user.length())
		{
			asRegText.cat_sprintf("%s@%s",
				appSettings.uaConf.accounts[id].user.c_str(),
				appSettings.uaConf.accounts[id].reg_server.c_str());
		}
		else
		{
			asRegText = appSettings.uaConf.accounts[id].reg_server.c_str();
		}
	}
	Item->SubItems->Add(asRegText);
	switch(reg_state[id])
	{
	case Callback::REG_STATE_INITIALIZING:
		asRegText = "Initializing";
		break;
	case Callback::REG_STATE_CONF_ERROR:
		asRegText = "Configuration error";
		break;
	case Callback::REG_STATE_NOREG:
		asRegText = "OK (no registration)";
		break;
	case Callback::REG_STATE_REGISTERING:
		asRegText = "Registering";
		break;
	case Callback::REG_STATE_REGISTER_OK:
		asRegText = "Registered";
		break;
	case Callback::REG_STATE_REGISTER_FAIL:
		asRegText = "Error registering";
		break;
	case Callback::REG_STATE_UNREGISTERING:
		asRegText = "Unregistering";
		break;
	case Callback::REG_STATE_UNREGISTER_OK:
		asRegText = "Unregistered";
		break;
	case Callback::REG_STATE_UNREGISTER_FAIL:
		asRegText = "Unregistering error";
		break;
	default:
		assert(!"Unhandled reg state");
		break;
	}
	Item->SubItems->Add(asRegText);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::lvAccountsChange(TObject *Sender, TListItem *Item,
      TItemChange Change)
{
	if (lvAccounts->Selected)
		prevSelectedAcc = lvAccounts->Selected;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::lvAccountsMouseDown(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y)
{
	if (!lvAccounts->Selected)
		lvAccounts->Selected = prevSelectedAcc;	
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::lvCallsData(TObject *Sender, TListItem *Item)
{
	int id = Item->Index;
	UaCalls &uaCalls = UaMain::Instance().uaCalls;
	if (id >= uaCalls.calls.size())
	{
		return;
	}
	UaCalls::CallData &call = uaCalls.calls[id];
	Item->Caption = call.callId + 1;
	Item->SubItems->Add(call.accId + 1);
	Item->SubItems->Add(CleanUri(call.uri));

	AnsiString asStateText;
	switch(call.state)
	{
	case Callback::CALL_STATE_INCOMING:
		asStateText = "Incoming call";
		break;
	case Callback::CALL_STATE_TRYING:
		asStateText = "Trying";
		break;
	case Callback::CALL_STATE_RINGING:
		asStateText = "Ringback";
		break;
	case Callback::CALL_STATE_PROGRESS:
		asStateText = "Call state progress";
		break;
	case Callback::CALL_STATE_ESTABLISHED:
		asStateText = "Connected";
		break;
	case Callback::CALL_STATE_CLOSED:
		asStateText = "End of call";
		break;
	default:
		assert(!"Unhandled call state");
		break;
	}
	Item->SubItems->Add(asStateText);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::btnAnswerClick(TObject *Sender)
{
	if (!lvCalls->Selected)
		return;
	int id = lvCalls->Selected->Index;
	UaCalls &uaCalls = UaMain::Instance().uaCalls;
	UA->Answer(uaCalls.calls[id].callId);
}
//---------------------------------------------------------------------------


void __fastcall TfrmMain::miClearFinishedCallsClick(TObject *Sender)
{
	UaMain::Instance().uaCalls.ReleaseFinished();
	lvCalls->Items->Count = UaMain::Instance().uaCalls.calls.size();
	lvCalls->Invalidate();	
}
//---------------------------------------------------------------------------

