//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "FormSettings.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TfrmSettings *frmSettings;
//---------------------------------------------------------------------------
__fastcall TfrmSettings::TfrmSettings(TComponent* Owner)
	: TForm(Owner)
{
	pcGeneral->ActivePage = tsGeneral;
}
//---------------------------------------------------------------------------
void __fastcall TfrmSettings::FormShow(TObject *Sender)
{
	tmpSettings = appSettings;

	cbSoundBackend->ItemIndex = tmpSettings.uaConf.soundBackend;

	edLocalAddress->Text = tmpSettings.uaConf.local.c_str();

	UpdateAccounts(tmpSettings.uaConf.accounts, 0);

	chbAlwaysOnTop->Checked = tmpSettings.frmMain.bAlwaysOnTop;

	chbLogToFile->Checked = tmpSettings.Logging.bLogToFile;
	chbLogMessages->Checked = tmpSettings.uaConf.logMessages;
	cmbMaxUiLogLines->ItemIndex = -1;
	for (int i=0; i<cmbMaxUiLogLines->Items->Count; i++)
	{
		if ((unsigned int)StrToInt(cmbMaxUiLogLines->Items->Strings[i]) >= tmpSettings.Logging.iMaxUiLogLines)
		{
			cmbMaxUiLogLines->ItemIndex = i;
			break;
		}
	}
	if (cmbMaxUiLogLines->ItemIndex == -1)
	{
		cmbMaxUiLogLines->ItemHeight = cmbMaxUiLogLines->Items->Count - 1;
	}

	chbG711->Checked = tmpSettings.uaConf.codecs.g711;
	chbG726_32->Checked = tmpSettings.uaConf.codecs.g726_32;
	chbGsm->Checked = tmpSettings.uaConf.codecs.gsm;
	chbSpeex->Checked = tmpSettings.uaConf.codecs.speex;
}

void TfrmSettings::UpdateAccounts(std::vector<UaConf::Account> &accounts, int visible)
{
    cbAccountSelector->OnChange = NULL;
	cbAccountSelector->Items->Clear();
	for (int i=0; i<accounts.size(); i++)
	{
		UaConf::Account& acc = accounts[i];
		AnsiString desc;
		desc.sprintf("#%02d: %s@%s", i+1, acc.user.c_str(), acc.reg_server.c_str());
		cbAccountSelector->Items->Add(desc);
	}

	cbAccountSelector->ItemIndex = visible;
	cbAccountSelector->OnChange = cbAccountSelectorChange;

	if (accounts.size() > 0)
	{
		UaConf::Account& acc = accounts[visible];

		edRegServer->Text = acc.reg_server.c_str();
		edUser->Text = acc.user.c_str();
		edAuthUser->Text = acc.auth_user.c_str();
		edPassword->Text = acc.pwd.c_str();
		cbTransport->ItemIndex = acc.transport;
		edExpires->Text = acc.reg_expires;

		edStunServer->Text = acc.stun_server.c_str();
	}
	else
	{
		edRegServer->Text = "";
		edUser->Text = "";
		edAuthUser->Text = "";
		edPassword->Text = "";
		cbTransport->ItemIndex = 0;
		edExpires->Text = 60;
		edStunServer->Text = "";
	}
}

void TfrmSettings::ApplyAccount(void)
{
	if (tmpSettings.uaConf.accounts.size() == 0) {
		UaConf::Account acc;
		tmpSettings.uaConf.accounts.push_back(acc);

		cbAccountSelector->OnChange = NULL;
		AnsiString desc;
		desc.sprintf("#%02d: %s@%s", 1, edUser->Text.c_str(), edRegServer->Text.c_str());
		cbAccountSelector->Items->Add(desc);

		cbAccountSelector->ItemIndex = 0;
		cbAccountSelector->OnChange = cbAccountSelectorChange;
	}
	UaConf::Account& acc = tmpSettings.uaConf.accounts[cbAccountSelector->ItemIndex];
	acc.reg_server = edRegServer->Text.c_str();
	acc.user = edUser->Text.c_str();
	acc.pwd = edPassword->Text.c_str();
	acc.transport = (UaConf::Account::TRANSPORT_TYPE)cbTransport->ItemIndex;
	acc.reg_expires = StrToIntDef(edExpires->Text, 60);
	acc.stun_server = edStunServer->Text.c_str();
	acc.auth_user = edAuthUser->Text.c_str();
}

//---------------------------------------------------------------------------
void __fastcall TfrmSettings::btnCancelClick(TObject *Sender)
{
	this->Close();	
}
//---------------------------------------------------------------------------
void __fastcall TfrmSettings::btnApplyClick(TObject *Sender)
{
    ApplyAccount();

	tmpSettings.Logging.bLogToFile = chbLogToFile->Checked;
	tmpSettings.uaConf.logMessages = chbLogMessages->Checked;
	tmpSettings.uaConf.soundBackend = (UaConf::SoundBackend)cbSoundBackend->ItemIndex;
	tmpSettings.uaConf.local = edLocalAddress->Text.c_str();

	tmpSettings.uaConf.codecs.g711 = chbG711->Checked;
	tmpSettings.uaConf.codecs.g726_32 = chbG726_32->Checked;
	tmpSettings.uaConf.codecs.gsm = chbGsm->Checked;
	tmpSettings.uaConf.codecs.speex = chbSpeex->Checked;

	appSettings = tmpSettings;
	this->Close();	
}
//---------------------------------------------------------------------------

void __fastcall TfrmSettings::chbAlwaysOnTopClick(TObject *Sender)
{
	tmpSettings.frmMain.bAlwaysOnTop = chbAlwaysOnTop->Checked;	
}
//---------------------------------------------------------------------------

void __fastcall TfrmSettings::cmbMaxUiLogLinesChange(TObject *Sender)
{
	tmpSettings.Logging.iMaxUiLogLines = StrToInt(cmbMaxUiLogLines->Text);	
}
//---------------------------------------------------------------------------

void __fastcall TfrmSettings::btnAddAccountClick(TObject *Sender)
{
    ApplyAccount();
	UaConf::Account acc;
	tmpSettings.uaConf.accounts.push_back(acc);
	int i = tmpSettings.uaConf.accounts.size() - 1;

	UpdateAccounts(tmpSettings.uaConf.accounts, i);	
}
//---------------------------------------------------------------------------


void __fastcall TfrmSettings::cbAccountSelectorChange(TObject *Sender)
{
	UpdateAccounts(tmpSettings.uaConf.accounts, cbAccountSelector->ItemIndex);	
}
//---------------------------------------------------------------------------

void __fastcall TfrmSettings::btnDeleteAccountClick(TObject *Sender)
{
	if (tmpSettings.uaConf.accounts.size() <= 1)
		return;
	int id = cbAccountSelector->ItemIndex;
	tmpSettings.uaConf.accounts.erase(tmpSettings.uaConf.accounts.begin() + id);
	if (id > 0)
	{
    	id--;
	}
	UpdateAccounts(tmpSettings.uaConf.accounts, id);
}
//---------------------------------------------------------------------------

