//---------------------------------------------------------------------------


#pragma hdrstop

#include "UaCalls.h"
#include "Mutex.h"
#include "ScopedLock.h"

//---------------------------------------------------------------------------

#pragma package(smart_init)

namespace {
	Mutex mutex;
	int lastCallId = -1;
}

UaCalls::UaCalls(void)
{
}

int UaCalls::AllocCallId(void)
{
	ScopedLock<Mutex> lock(mutex);
	++lastCallId;
	if (lastCallId < 0)
		lastCallId = 0;
	return lastCallId;
}

extern "C" int alloc_call_id(void)
{
	ScopedLock<Mutex> lock(mutex);
	++lastCallId;
	if (lastCallId < 0)
		lastCallId = 0;
	return lastCallId;
}

UaCalls::CallData* UaCalls::AllocCall(int callId, int accId, AnsiString uri, enum Dir dir, Callback::ua_state_e state)
{
	ScopedLock<Mutex> lock(mutex);
	CallData data;
	data.callId = callId;
	data.accId = accId;
	data.uri = uri;
	data.dir = dir;
	data.state = state;
	calls.push_back(data);
	CallData *ptr = &calls[calls.size()-1];
	return ptr;
}

UaCalls::CallData* UaCalls::FindCall(int callId)
{
	ScopedLock<Mutex> lock(mutex);
	for (unsigned int i=0; i<calls.size(); i++)
	{
		if (calls[i].callId == callId)
			return &calls[i];
	}
	return NULL;
}

int UaCalls::SetState(int callId, Callback::ua_state_e state)
{
	ScopedLock<Mutex> lock(mutex);
	for (unsigned int i=0; i<calls.size(); i++)
	{
		if (calls[i].callId == callId)
		{
			calls[i].state = state;
			return 0;
		}
	}
	return -1;
}

void UaCalls::ReleaseCall(int callId)
{
	ScopedLock<Mutex> lock(mutex);
	for (unsigned int i=0; i<calls.size(); i++)
	{
		if (calls[i].callId == callId)
		{
			calls.erase(&calls[i]);
			return;
		}
	}
}

void UaCalls::ReleaseFinished(void)
{
	// clever way of removing vector elements
	size_t j = 0;
	for (size_t i = 0; i < calls.size(); ++i)
	{
		if (calls[i].state != Callback::CALL_STATE_CLOSED)
		{
			calls[j++] = calls[i];
		}
	}
	// trim vector to new size
	calls.resize(j);
}

